import getNumberName from '../../../index';
import getLocalizedNumberName from './index';

describe('Landon\'s original test cases', () => {
  describe('Basic conversions', () => {
    it.each`
      value                  | traditionalEuropeanName
      ${1}                   | ${'ein'}
      ${1000}                | ${'eintausend'}
      ${1000000}             | ${'eine Million'}
      ${1000000000}          | ${'eine Milliarde'}
      ${1000000000000}       | ${'eine Billion'}
      ${1000000000000000}    | ${'eine Billiarde'}
      ${1000000000000000000} | ${'eine Trillion'}
    `('converts $value to $traditionalEuropeanName', ({value, traditionalEuropeanName}) => {
      expect(getNumberName(value, {locale: getLocalizedNumberName})).toBe(traditionalEuropeanName);
    });
  });

  describe('Medium size numbers (<= 1e+63)', () => {
    describe('Table 1', () => {
      it.each`
        value      | traditionalEuropeanName
        ${'1e+9'}  | ${'Milliarde'}
        ${'1e+12'} | ${'Billion'}
        ${'1e+15'} | ${'Billiarde'}
        ${'1e+18'} | ${'Trillion'}
        ${'1e+21'} | ${'Trilliarde'}
        ${'1e+24'} | ${'Quadrillion'}
        ${'1e+27'} | ${'Quadrilliarde'}
        ${'1e+30'} | ${'Quintillion'}
        ${'1e+33'} | ${'Quintilliarde'}
        ${'1e+36'} | ${'Sextillion'}
        ${'1e+39'} | ${'Sextilliarde'}
        ${'1e+42'} | ${'Septillion'}
        ${'1e+45'} | ${'Septilliarde'}
        ${'1e+48'} | ${'Octillion'}
        ${'1e+51'} | ${'Octilliarde'}
        ${'1e+54'} | ${'Nonillion'}
        ${'1e+57'} | ${'Nonilliarde'}
        ${'1e+60'} | ${'Decillion'}
        ${'1e+63'} | ${'Decilliarde'}
      `('converts $value to $traditionalEuropeanName', ({value, traditionalEuropeanName}) => {
        expect(getNumberName(value, {locale: getLocalizedNumberName})).toBe(`eine ${traditionalEuropeanName}`);
      });
    });
  });
});